Documentation.addTranslations({"locale": "fi", "messages": {"%(filename)s &#8212; %(docstitle)s": "", "&#169; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "", "&#169; Copyright %(copyright)s.": "", ", in ": "", "About these documents": "Tietoja t\u00e4st\u00e4 documentist\u00e4", "Automatically generated list of changes in version %(version)s": "Automaattisesti luotu muutoshistoria alkaen versiosta %(version)s", "C API changes": "", "Changes in Version %(version)s &#8212; %(docstitle)s": "", "Collapse sidebar": "", "Complete Table of Contents": "", "Contents": "", "Copyright": "", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "", "Expand sidebar": "", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "Anna hakusanat kokonaan, osasanoilla ei haeta.", "Full index on one page": "Hakemisto yhten\u00e4 luettelona", "General Index": "Yleinen sis\u00e4llysluettelo", "Global Module Index": "Yleinen moduulien sis\u00e4llysluettelo", "Go": "Siirry", "Hide Search Matches": "Piilota l\u00f6ydetyt", "Index": "Sis\u00e4llysluettelo", "Index &ndash; %(key)s": "", "Index pages by letter": "Hakemisto aakkostus sivuttain", "Indices and tables:": "", "Last updated on %(last_updated)s.": "", "Library changes": "", "Navigation": "Navikointi", "Next topic": ">>", "Other changes": "", "Overview": "Yhteenveto", "Permalink to this definition": "", "Permalink to this headline": "", "Please activate JavaScript to enable the search\n    functionality.": "Javascript pit\u00e4\u00e4 olla sallittu, jotta etsint\u00e4 toimii.", "Preparing search...": "", "Previous topic": "<<", "Quick search": "Pikahaku", "Search": "Etsi", "Search Page": "Etsi sivu", "Search Results": "Etsinn\u00e4n tulos", "Search finished, found %s page(s) matching the search query.": "", "Search within %(docstitle)s": "", "Searching": "", "Show Source": "N\u00e4yt\u00e4 l\u00e4hdekoodina", "Table Of Contents": "Sis\u00e4llysluettelo", "This Page": "T\u00e4m\u00e4 sivu", "Welcome! This is": "", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "", "all functions, classes, terms": "", "can be huge": "voi olla iso", "last updated": "", "lists all sections and subsections": "", "next chapter": ">>", "previous chapter": "<<", "quick access to all modules": "", "search": "etsi", "search this documentation": "", "the documentation for": ""}, "plural_expr": "(n != 1)"});