Documentation.addTranslations({"locale": "sv", "messages": {"%(filename)s &#8212; %(docstitle)s": "", "&#169; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "", "&#169; Copyright %(copyright)s.": "", ", in ": "", "About these documents": "Om dessa dokument", "Automatically generated list of changes in version %(version)s": "Automatiskt genererad lista \u00f6ver f\u00f6r\u00e4ndringar i version %(version)s", "C API changes": "F\u00f6r\u00e4ndringar i C-API", "Changes in Version %(version)s &#8212; %(docstitle)s": "", "Collapse sidebar": "D\u00f6lj sidolist", "Complete Table of Contents": "Komplett Inneh\u00e5llsf\u00f6rteckning", "Contents": "Inneh\u00e5ll", "Copyright": "Copyright", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "Skapad med <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.", "Expand sidebar": "Expandera sidolist", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "H\u00e4r kan du s\u00f6ka bland dessa dokument. Ange s\u00f6kord nedan och klicka \"s\u00f6k\".\n    S\u00f6kningen m\u00e5ste tr\u00e4ffa p\u00e5 samtliga angivna s\u00f6kord.", "Full index on one page": "Hela inneh\u00e5llsf\u00f6rteckningen p\u00e5 en sida", "General Index": "Huvudindex", "Global Module Index": "Global Modulindex", "Go": "G\u00e5", "Hide Search Matches": "D\u00f6lj S\u00f6kresultat", "Index": "Index", "Index &ndash; %(key)s": "Index &ndash; %(key)s", "Index pages by letter": "Inneh\u00e5llsf\u00f6rteckning per inledande bokstav", "Indices and tables:": "Index och tabeller", "Last updated on %(last_updated)s.": "Senast uppdaterad %(last_updated)s.", "Library changes": "F\u00f6r\u00e4ndringar i bibliotek", "Navigation": "Navigation", "Next topic": "N\u00e4sta titel", "Other changes": "\u00d6vriga f\u00f6r\u00e4ndringar", "Overview": "\u00d6versikt", "Permalink to this definition": "Permalink till denna definition", "Permalink to this headline": "Permalink till denna rubrik", "Please activate JavaScript to enable the search\n    functionality.": "Var god aktivera JavaScript f\u00f6r s\u00f6kfunktionalitet.", "Preparing search...": "", "Previous topic": "F\u00f6reg\u00e5ende titel", "Quick search": "Snabbs\u00f6k", "Search": "S\u00f6k", "Search Page": "S\u00f6ksida", "Search Results": "S\u00f6kresultat", "Search finished, found %s page(s) matching the search query.": "", "Search within %(docstitle)s": "S\u00f6k bland %(docstitle)s", "Searching": "", "Show Source": "Visa k\u00e4llfil", "Table Of Contents": "Inneh\u00e5llsf\u00f6rteckning", "This Page": "Denna Sida", "Welcome! This is": "", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "", "all functions, classes, terms": "alla funktioner, klasser, villkor", "can be huge": "kan bli stort", "last updated": "", "lists all sections and subsections": "lista \u00f6ver alla paragrafer och underparagrafer", "next chapter": "N\u00e4sta kapitel", "previous chapter": "F\u00f6reg\u00e5ende kapitel", "quick access to all modules": "genv\u00e4g till alla moduler", "search": "s\u00f6k", "search this documentation": "s\u00f6k i det h\u00e4r dokumentet", "the documentation for": ""}, "plural_expr": "(n != 1)"});